# Bryan Schumaker (11/12/2010)

import libsaria
from libsaria import path
from libsaria.sources import library
import libsaria.xm as xml

add_child = xml.add_child
add_text  = xml.add_text
get_attrs = library.get_attrs
set_attr  = library.set_attr
file2id   = library.file_to_id
lib_file  = "library"
vers      = "%s.%s" % (libsaria.__major__, libsaria.__minor__)
if libsaria.__dev__ == True:
	lib_file = lib_file + "-dev"
lib_file += ".xml"
lib_file  = path.join(path.saria_dir(), lib_file)

xml_children = xml.children
xml_child    = xml.child
xml_attrs    = xml.attributes


def encode_attr(doc, node, attr, value):
	child = add_child(doc, node, attr)
	add_text(doc, child, value)


def decode_attr(attr):
	child = xml.child(attr)
	return (attr.nodeName, child.data.strip())


def encode_track(doc, node, id):
	filepath, score, count = get_attrs(id, "filepath", "score", "count")
	if score == 0 and count == 0:
		return
	child = add_child(doc, node, "track", {"score":score, "count":count})
	add_text(doc, child, filepath)


def decode_sources_v1_2(node):
	for child in xml_children(node):
		source = xml_child(child).data.strip()
		print "Scanning source: %s" % source
		libsaria.sources.new_source(source)


def decode_track_v1_1(node):
	track = dict()
	for child in xml_children(node):
		key, value = decode_attr(child)
		track[key] = value
	id = file2id(track["filepath"])
	set_attr(id, "count", int(track["count"]))
	set_attr(id, "score", int(track["score"]))


def decode_track_v1_2(node):
	for track in xml_children(node):
		attrs      = xml_attrs(track)
		name, path = decode_attr(track)
		id = file2id(path)
		set_attr(id, "count", int(attrs["count"]))
		set_attr(id, "score", int(attrs["score"]))


def decode_v1_1(node):
	for track in xml_children(node):
		decode_track_v1_1(track)


def decode_v1_2(node):
	children = xml_children(node)
	sources  = None
	tracks   = None
	for node in children:
		name = node.nodeName
		if name == "sources":
			sources = node
		elif name == "tracks":
			tracks = node
	decode_sources_v1_2(sources)
	decode_track_v1_2(tracks)


def backup():
	doc     = xml.new()
	child   = add_child(doc, doc, "library", {"version":vers})
	sources = add_child(doc, child, "sources")
	tracks  = add_child(doc, child, "tracks")
	for source in library.get_sources():
		encode_attr(doc, sources, "source", source)

	for id in library.walk():
		encode_track(doc, tracks, id)

	fout = open(lib_file, 'w')
	fout.write(doc.toprettyxml())
	fout.close()
	print "Wrote library to: %s" % lib_file


def restore(file):
	fin   = open(file)
	doc   = xml.parse(fin)
	child = xml.child(doc)
	attrs = xml.attributes(child)
	version = attrs.get("version", "1.1")
	print "Decoding version: %s" % version
	if version == "1.1":
		decode_v1_1(child)
	elif version == "1.2":
		decode_v1_2(child)
